import type * as GeoJSON from 'geojson';
import type { MapDimensions, Region } from './types';
import type Supercluster from './types';
export default class SuperclusterClass<P extends GeoJSON.GeoJsonProperties = Supercluster.AnyProps, C extends GeoJSON.GeoJsonProperties = Supercluster.AnyProps> {
    private cppInstance;
    private options;
    constructor(options?: Supercluster.Options<P, C>);
    /**
     * Loads an array of GeoJSON Feature objects. Each feature's geometry
     * must be a GeoJSON Point. Once loaded, index is immutable.
     *
     * @param points Array of GeoJSON Features, the geometries being GeoJSON Points.
     */
    load(points: Array<Supercluster.PointFeature<P>>): SuperclusterClass<P, C>;
    /**
     * Returns an array of clusters and points as `GeoJSON.Feature` objects
     * for the given bounding box (`bbox`) and zoom level (`zoom`).
     *
     * @param bbox Bounding box (`[westLng, southLat, eastLng, northLat]`).
     * @param zoom Zoom level.
     */
    getClusters(bbox: GeoJSON.BBox, zoom: number): Array<Supercluster.ClusterFeature<C> | Supercluster.PointFeature<P>>;
    /**
     * Returns an array of clusters and points as `GeoJSON.Feature` objects
     * for the given bounding box (`bbox`) and zoom level (`zoom`).
     *
     * @param bbox Bounding box (`[westLng, southLat, eastLng, northLat]`).
     * @param zoom Zoom level.
     */
    getClustersFromRegion(region: Region, mapDimensions: MapDimensions): Array<Supercluster.ClusterFeature<C> | Supercluster.PointFeature<P>>;
    /**
     * For a given zoom and x/y coordinates, returns a
     * [geojson-vt](https://github.com/mapbox/geojson-vt)-compatible JSON
     * tile object with cluster any point features.
     */
    getTile(x: number, y: number, zoom: number): Supercluster.Tile<C, P> | null;
    /**
     * Returns the children of a cluster (on the next zoom level).
     *
     * @param clusterId Cluster ID (`cluster_id` value from feature properties).
     * @throws {Error} If `clusterId` does not exist.
     */
    getChildren(clusterId: number): Array<Supercluster.ClusterFeature<C> | Supercluster.PointFeature<P>>;
    /**
     * Returns all the points of a cluster (with pagination support).
     *
     * @param clusterId Cluster ID (`cluster_id` value from feature properties).
     * @param limit The number of points to return (set to `Infinity` for all points).
     * @param offset The amount of points to skip (for pagination).
     */
    getLeaves(clusterId: number, limit?: number, offset?: number): Array<Supercluster.PointFeature<P>>;
    /**
     * Returns the zoom level on which the cluster expands into several
     * children (useful for "click to zoom" feature).
     *
     * @param clusterId Cluster ID (`cluster_id` value from feature properties).
     */
    getClusterExpansionZoom(clusterId: number): number;
    /**
     * Returns a region containing the center of all the points in a cluster
     * and the delta value by which it should be zoomed out to see all the points.
     * (usefull for animating a MapView after a cluster press).
     * @param clusterId Cluster ID (`cluster_id` value from feature properties).
     */
    expandCluster: (clusterId: number) => Region;
    /**
     * Destroy the instance.
     * @deprecated Removed in 1.2.0.
     * @returns True if cluster exists and was destroyed, else false.
     */
    destroy(): boolean;
    private getMarkersInCluster;
}
